"use strict";

/*
 * @version 1.0.0
 */

/**
 * Loads dynamically, i.e. JavaScript triggered, JavaScript and style sheet 
 * files.
 * 
 * @type CookieQuestionDynLoader
 */
class CookieQuestionDynLoader {

    /**
     * Loads all scripts denoted by arguments in reverse order, if any. Scripts
     * are inserted at the end of the head section.
     * 
     * @param {function} callback The function to call after all scripts have 
     * been inserted.
     * @param {Array} An array of Strings denoting the JavaScript files (file 
     * path names) to load in reverse order. For each script calls itself.
     * 
     * Typical use in webpage:
     * <script src="CookieQuestionDynLoader.js"></script>
     * CookieQuestionDynLoader.loadScriptsHeadEndReverse(pageStartFunction.bind(this), ["javaScript9.js", "javaScript8.js", ...]);
     * function pageStartFunction() {...}
     * 
     * Typical use in separate script file:
     * webpage: 
     * <script src="CookieQuestionDynLoader.js"></script>
     * <script src="scriptFile.js"></script>
     * script file:
     * CookieQuestionDynLoader.loadScriptsHeadEndReverse(
     * pageStartFunction.bind(this),
     * function() {
     *  init(); // start java script using loaded files ...
     * }
     *  ["javaScript9.js", "javaScript8.js", ...]);
     *  where the CookieQuestionDynLoader inclusion can be omitted in webpage if scriptFile.js
     *  loads this CookieQuestionDynLoader.js "dynamic manually" (like below) first.
     *  
     * HINT: The charset attibute is always set to "utf8", so the files have to
     * be encoded in utf8.
     * 
     * HINT: IS USABLE in separate script file.
     * 
     * TODO: Test if: HINT: NOT USABLE WITH InternetExplorer! 
     * 
     * @returns {undefined}
     */
    static loadScriptsHeadEndReverse(callback, scriptNamesArray) {
        // create a single plain array of all args
        if (scriptNamesArray.length) {
            var scriptName = scriptNamesArray.pop();
            var scriptElem = document.createElement("script");
            scriptElem.type = "text/javascript";
            scriptElem.charset = "utf8";
            scriptElem.async = false;
            scriptElem.defer = false;
            scriptElem.src = scriptName;
            scriptElem.onload = function () {
                window.setTimeout(CookieQuestionDynLoader.loadScriptsHeadEndReverse, 10, callback, scriptNamesArray);
            };
            document.getElementsByTagName("head")[0].appendChild(scriptElem);
        }
        else {
            window.setTimeout(callback, 10);
        }
    }

    /**
     * Dynamically loads a stylesheet. Does not set any properties
     * other than rel, type, href and id. This function may not return 
     * synchromously if document.getElementsByTagName('head')[0] is not
     * currently accessable.
     * @param {String|Array} cssFileUrl(s) The absolute or relative URL(s) of 
     * the stylesheet(s) to load.
     * to load.
     * @param {string|Array} linkElementId the element id to set, also set if 
     * given as empty string. If given not empty a preemptive existing element 
     * with that same id is removed from the document first. Optional, defaults
     * to the empty string. If cssFileUrl is an array of url's, then 
     * linkElementId is converted into an array, if it is not, and that array is
     * filled until cssFileUrl size with empty strings.
     * @returns {Node} The (last) newly created link element, setup to refer to 
     * cssFileUrl, inserted as last current child of the head node.
     */
    static loadStyles(cssFileUrl = "", linkElementId = "") {
        var head = document.getElementsByTagName('head')[0];
        if (head === null) {
            window.setTimeout(CookieQuestionDynLoader.loadStyles, 100, cssFileUrl, linkElementId);
        }
        else {
            var styleLinkNode;
            
            var urlArray = (typeof(cssFileUrl) === "string") ? cssFileUrl.split(/[]/g) : cssFileUrl;
            var idArray = (typeof(linkElementId) === "string") ? linkElementId.split(/[]/g) : linkElementId;
            var len = urlAttay.length;
            var nrIds = idArray.length;
            if(nrIds < len) {
                tabBlanks = new Array(len - nrIds);
                tabBlanks.fill(" ");
                Array.prototype.push.apply(idArray, tabBlanks);
            }
            var styleLinkNode;
            for(var i=0; i<len; i++) {
                var url = urlArray[i];
                var id = idArray[i];
                styleLinkNode = document.getElementById(id);
                if (styleLinkNode) {
                    styleLinkNode.parentNode.removeChild(styleLinkNode);
                    styleLinkNode = null;
                }
                styleLinkNode = document.createElement("link");
                styleLinkNode.id = id;
                styleLinkNode.rel = "stylesheet";
                styleLinkNode.type = "text/css";
                styleLinkNode.href = url;
                head.appendChild(styleLinkNode);
            }
            return styleLinkNode;
        }
    }
}
