
"use strict";
/*
 * 
 * @version 1.0.0
 */

/**
 * Class to test and convert strings denoting IPv4 addresses.
 * 
 * Include, e.g., like 
 * <script charset="UTF-8" src="file:///usr/local/JavaScript/CookieQuestionIp4Address.js">
 * @type CookieQuestionIp4Address
 */
class CookieQuestionIp4Address {
    /**
     * @constructor
     * @returns {CookieQuestionIp4Address}
     */
    constructor () {
    }

    /**
     * To be called CookieQuestionIp4Address.ip4AddrArray(...).
     * 
     * Tests the input for denoting a valid ipv4 address String and converts it
     * into an array of 4 numbers.
     * @param {variant} ip4AddrStr a string in format [n[.n[.n[.n]]]][.UNRECOGNIZED_POSTFIX],
     * with each n denoting a number in range 0..255. The String() converted
     * value will be used. Different parts must be delimited by a dot ('.'). 
     * Empty parts, i.e. a dot without prefix and/or postfix, as e.g. in
     * ip4AddrStr = '.123.' will be taken as defaultVal. Optional, defaults to
     * the empty string.
     * @param {integer|string denoting number in range 0..255} defaultVal The 
     * number to use for ungiven parts/octets. Optional, defaults to zero. 
     * Causes an empty return if not in range.
     * @param {integer|string number in range 0..4} minOctets denotes the 
     * minimum number of (non empty) parts/octets required to accept ip4AddrStr
     * as valid represantation of an ipv4 address. Optional, defaults to 0.
     * Causes an empty return if not in range.
     * @return Array An array of 4 numbers, each denoting an integer number in 
     * range 0..255, an empty array if ip4AddrStr does not denote an ipv4 address. E.g.
     * ip4AddrArray('123.45', 99, 1) will return [123,45,99,99],
     * ip4AddrArray("..11", 255, 0) will return [255,255,11,255],
     * ip4AddrArray() will return [0,0,0,0],
     */
    static ip4AddrArray(ip4AddrStr = "", defaultVal = 0, minOctets = 0) {
        var ipv4Array = new Array();
        var defaultStr = String(defaultVal);
        var defaultNum = parseInt(defaultStr);
        var minNrParts = parseInt(String(minOctets));
        if ((defaultNum >= 0) && (defaultNum <= 255) && (minNrParts >= 0) && (minNrParts <= 4)) {
            var as = String(ip4AddrStr);
            if (typeof (as) === 'string') {
                var aa = (as.length === 0) ? [] : as.split('.');
                var len = aa.length;
                if (len >= minNrParts) {
                    var ok = true;
                    for (var i = 0; ok && (i < 4); i++) {
                        var byteStr = ((i >= len) || (aa[i].length === 0)) ? defaultStr : aa[i];
                        ok = byteStr.match(/^\d{1,3}$/);
                        if (ok) {
                            var byteVal = parseInt(byteStr);
                            ok = ((byteVal >= 0) && (byteVal <= 255));
                            if (ok) {
                                ipv4Array.push(byteVal);
                            }
                        }
                    }
                    if (ok) {
                        for (var i = ipv4Array.length; i < 4; i++) {
                            ipv4Array.push(defaultNum);
                        }
                    }
                    else {
                        // unvalid part(s), empty the array
                        ipv4Array.splice(0, ipv4Array.length);
                    }
                }
            }
        }
        return ipv4Array;
    }

    /**
     * To be called CookieQuestionIp4Address.ip4AddrArray(...).
     * 
     * Tests the input for denoting a valid ipv4 address, optionally inside a
     * given address range.
     * @param {string} ip4AddrStr a string in format [n[.n[.n[.n]]]][.UNRECOGNIZED_POSTFIX],
     * with each n denoting a number in range 0..255. The String() converted
     * value will be used. Different parts must be delimited by a dot ('.'). 
     * Empty parts, i.e. a dot without prefix and/or postfix, as e.g. in
     * ip4AddrStr = '.123.' will be taken as zero. Optional, defaults to
     * the empty string.
     * @param {array of 4 integer numbers} min The lower bound of the accepted 
     * address range, array elements with values less than zero cause the same 
     * effect as zero. Optional, defaults to [0,0,0,0].
     * @param {array of 4 integer numbers} max The upper bound of the accepted 
     * address range, array elements with values higher than 255 cause the same
     * effect as 255. Optional, defaults to [255,255,255,255].
     * @param {integer|string number in range 0..4} minOctets denotes the 
     * minimum number of (non empty) parts/octets required to accept ip4AddrStr
     * as valid represantation of an ipv4 address. Optional, defaults to 0.
     * Causes an empty return if not in range.
     * @return true if ip4AddrStr denotes a valid ipv4 address and min starts 
     * with at least 4 numbers and max starts with at least 4 numbers and 
     * minOctets is in range, false if not.
     */
    static isIp4AddrInRange(ip4AddrStr = "", min = [0, 0, 0, 0], max = [255, 255, 255, 255], minOctets = 0) {
        var numArray = CookieQuestionIp4Address.ip4AddrArray(ip4AddrStr, 0, minOctets);
        var isIp4 = false;
        action: {
            if (numArray.length === 4) {
                for (var i = 0; i < 4; i++) {
                    if ((typeof(min[i]) !== "number") || (typeof(max[i]) !== "number") || (numArray[i] < min[i]) || (numArray[i] > max[i])) {
                        break action;
                    }
                }
                isIp4 = true;
            }
        }
        return isIp4;
    }

}
