"use strict";

/*
 * 
 * @version 1.0.0
 */

/**
 * 
 * @type CookieQuestionBlendHelper
 */
class CookieQuestionBlendHelper {

    /**
     * Sets the opacity of an element node.
     * 
     * Hint: HTMLElements to set opcaity of must have the opacity explicitely
     * set, but not using a rgba(r,g,b,opacity) value.
     * <p>
     * Hint: The Konqueror browser version 3.5 does not support opacity, but earlier
     * and later versions may do.
     * <p>
     * Hint: Blending does not work for inline elements, but for containers
     * containing such.
     * <p>
     * Hint: Browser which set opacity via the MozOpacity property (old Gecko's)
     * cannot blend table-, but div-elements containing such.
     * <p>
     * Hint: For IE (6) elements need to have a (true) "hasLayout" property. This
     * can be achieved by assigning them a style-width or -height with a different
     * value than "auto", e.g. height=100%.
     *
     * @param {HTMLElement} blendElem The value of the id attribute of the
     * element node to set the opacity of.
     * @param {Number} opacityPercentage An integer value in the range 0..100,
     * specifying the opacity to set. Zero means "unvisibilty", i.e. full
     * transparence, 100 means full visibility, i.e. no transparence.
     * @return {undefined}
     */
    static setObjectOpacity(blendElem, opacityPercentage) {
        var scale = Number(opacityPercentage) / 100; /* value 1.0 makes Netscape 8 flicker */
        /* IE */
        if (blendElem.style && blendElem.style.filter) {
            blendElem.style.filter = "alpha(opacity=" + String(opacityPercentage) + ")";
        }
        if (blendElem.style) {
            if (blendElem.style.MozOpacity)
            {
                blendElem.style.MozOpacity = scale;
                return;
            }
            /* Konqueror (Safari) */
            if (blendElem.style.KhtmlOpacity) {
                blendElem.style.KhtmlOpacity = scale;
            }
            /* common */
            if (blendElem.style.opacity) {
                blendElem.style.setProperty("opacity", scale);
            }
        }
    }

    /**
     * Dynamically changes the opacity of an HTMLElement.
     * 
     * Hint: HTMLElements to blend in or out must have the opacity explicitely
     * set, but not using a rgba(r,g,b,opacity) value.
     * 
     * @param {HTMLElement} blendElem The element to blend out or in.
     * @param {Number} destOpacityPercentage A numbe in range [0, 100] that
     * denotes the final opacity of the element after the blend process. Zero
     * means full transparency, 100 means full visibility.
     * @param {Number} blendPercentageStep A positive or negative number, sign
     * is ignored, that denotes the opacity difference percentage per blend 
     * step. Senseful values are in range [1, 99]. Optional, default is 1.0.
     * @param {Number} blendStepDelayMs The ms delay between the single blend
     * steps. Optional, default is 20.
     * @param {function} callback The function to call after blendElem has
     * reached it's destOpacityPercentage. Optional, the default is to not
     * callback.
     * @returns {undefined}
     */
    static blend(blendElem, destOpacityPercentage, blendPercentageStep = 1.0, blendStepDelayMs = 20, callback = function(){}) {
        if (blendElem) {
            var currentOpacityPercentage = blendElem.style.getPropertyValue("opacity") * 100.0;
            var opaDif = destOpacityPercentage - currentOpacityPercentage;
            if (Math.abs(opaDif) < 0.0001) {
                    callback();
            }
            else {
                var lastStep = true;
                if (Math.abs(opaDif) > Math.abs(blendPercentageStep)) {
                    // IE problem: opaDif = Math.sign(opaDif) * Math.abs(blendPercentageStep);
                    opaDif = (opaDif < 0 ? -1 : (opaDif > 0 ? 1 : 0)) * Math.abs(blendPercentageStep);
                    lastStep = false;
                }
                var newOpaPercent = currentOpacityPercentage + opaDif;
                CookieQuestionBlendHelper.setObjectOpacity(blendElem, newOpaPercent);
                //blendElem.style.setProperty("opacity", (newOpaPercent/100));
                if (!lastStep) {
                    window.setTimeout(CookieQuestionBlendHelper.blend, blendStepDelayMs, blendElem, destOpacityPercentage, blendPercentageStep, blendStepDelayMs, callback);
                }
                else {
                        callback();
                }
            }
        }

    }
}
