
"use strict";

/*
 * 
 * @version 1.0.0
 */

/**
 * Utility functions to create a new HTMLElement and/or apply css styles on.
 * @type CookieQuestionStyleElement
 */
class CookieQuestionStyleElement {

    /**
     * Creates a new HTMLElement or gets on from the current document.
     * @param {string|HtmlElement} htmlElement Optional, specifies the 
     * HTMLElement.
     * If given an existing HTMLElement then that is returned unchanged.
     * If given as string starting with "#" (hash mark), then it is interpreted
     * as the id of an HTMLElement in the document and that element is used.
     * If given as string not starting with "#" (hash mark), then it is 
     * interpreted as the type of element to newly create like "p", "span", 
     * "div", etc.
     * If ungiven or failed to get, then a new div element is created.
     * @returns {HTMLElement} The HtmlElement as specified by htmlElement.
     */
    static getHtmlElement(htmlElement = "div") {
        var elem = htmlElement;
        var heType = typeof(elem);
        if(heType === "string") {
            if (elem.substr(0,1) === "#") {
                // get by id
                if (elem.length > 1) {
                    elem = document.getElementById(elem.substr(1));
                }
            }
            else if (elem) {
                // create new
                elem = document.createElement(htmlElement);
            }
        }
        // fallback: create new div
        if((typeof(elem) !== "object") || !(elem instanceof HTMLElement)) {
            elem = document.createElement("div");
        }
        return elem;
    }
    
    /**
     * Applies style definitions to an existing or newly created HTMLElement.
     * Hint preemtive styles stay untouched as long they are not overwritten by 
     * css.
     * @param {string|Array} css 
     * Either A String containing css notations in kebab or camelCase style,
     * delimited by semicolon, on second level by colon.
     * or an array of strings, each containing a single speparate css notation
     * in kebab or camelCase style, attribure and value delimited by colon.
     * @param {string|HtmlElement} htmlElement Optional, specifies the 
     * HTMLElement.
     * If given an existing HTMLElement then css is applied to that.
     * If given as string starting with "#" (hash mark), then it is interpreted
     * as the id of an HTMLElement in the document and that element is used.
     * If given as string not starting with "#" (hash mark), then it is 
     * interpreted as the type of element to newly create like "p", "span", 
     * "div", etc.
     * If ungiven or failed to get, then a new div element is created.
     * @param {string} id The id of the element to set. Optional, if ungiven
     * or empty no id is set.
     * @returns {HTMLElement} The htmlElement with css applied.
     */
    static applyCss(css = "", htmlElement = "div", id="") {
        var elem = CookieQuestionStyleElement.getHtmlElement(htmlElement);
        var cssType = typeof(css);
        var cssArray;
        if(cssType === "string") {
            cssArray = css.split(";");
        }
        else if(css instanceof Array) {
            cssArray = css;
        }
        if(typeof(cssArray !== "undefined")) {
            for(var k in cssArray) {
                var vStyle = cssArray[k].trim();
                if(vStyle.length) {
                    var firstColonPos = vStyle.indexOf(":");
                    if(firstColonPos > 0) {
                        var attr = CookieQuestionStyleElement.kebabLowerCamel(vStyle.substring(0, firstColonPos));
                        var val = vStyle.substring(firstColonPos + 1).trim();
                        if(val && (val.substr(val.length - 1) === ";")) {
                            val = val.substr(0, val.length - 1).trim();
                        }
                        if(attr && val) {
                            elem.style[attr] = val;
                        }
                    }
                }
            }
        }
        if(id) {
            elem.id = id;
        }
        return elem;
    }
    
    /**
     * Appends text (nodes) to an HTMLElement.
     * Links are supported: If an element of the text parameter has the format
     * ">>>[link destination uri]<<<[link text]" then [link text] is appended as
     * link to [link destination uri] with target "_blank".
     * @param {Array} text the text to append, for each non empty element (text)
     * a text node with that content will be appended as child node, empty
     * string elements will each be appended as a "br" element.
     * @param {string|HtmlElement} htmlElement Optional, specifies the 
     * HTMLElement.
     * If given an existing HTMLElement then that is returned unchanged.
     * If given as string starting with "#" (hash mark), then it is interpreted
     * as the id of an HTMLElement in the document and that element is used.
     * If given as string not starting with "#" (hash mark), then it is 
     * interpreted as the type of element to newly create like "p", "span", 
     * "div", etc.
     * If ungiven or failed to get, then a new div element is created.
     * @returns {CookieQuestionStyleElement.appendText.elem|HTMLElement|undefined}
     */
    static appendText(text = [], htmlElement = "div") {
        var elem = CookieQuestionStyleElement.getHtmlElement(htmlElement);
        var linkNr = 0;
        for (var k in text) {
            var msg = text[k];
            if(msg.indexOf(">>>") === 0) {
                var linkMsg = msg.substr(3).split("<<<");
                if(linkMsg.length === 2) {
                    // link
                    var aElm = document.createElement("a");
                    aElm.href = linkMsg[0];
                    aElm.target = "_blank";
                    aElm.id = "cookieQuestionMsgLink_"+(linkNr += 1)+"_id"
                    aElm.appendChild(document.createTextNode(linkMsg[1]));
                    elem.appendChild(aElm);
                    continue;
                }
            }
            elem.appendChild( msg ? document.createTextNode(msg) : document.createElement("br"));
        }
        return elem;
    }

    /**
     * Appends a style sheet to the end of the document head and applies
     * the given style sheet rules to it.
     * @param {Array} styleRules An Array of strings. All elements must denote
     * a complete single stylesheet definition and are applied in the given 
     * order. The single rules must look like, e.g.,
     * "div { color: red; }", 
     * "a:hover, a.homelink:focus: { color: red; border-width 2px; }",
     * etc.
     * @returns {StyleSheet} the style sheet created.
     */
    static createStyleSheet(styleRules = []) {
        var style = document.createElement("style");
        style.appendChild(document.createTextNode(""));
        document.head.appendChild(style);
        var styleSheet = style.sheet;
        for(var k in styleRules) {
            styleSheet.insertRule(styleRules[k], styleSheet.cssRules.length);
        }
        return style;
    }
    
    /**
     * Removes a style sheet from document.head.
     * 
     * @param {StyleSheet} styleElem the style sheet to remove.
     * @returns {undefined}
     */
    static removeStyleSheet(styleElem) {
        document.head.removeChild(styleElem);
    }
    
    /**
     * Converts a string into lower camelCase.
     * @param {string} kebabWord a word in kebab-style
     * @returns {string} kebeabWord converted into camelCase
     */
    static kebabLowerCamel(kebabWord = "") {
        var camel = "";
        var parts = kebabWord.split("-");
        var firstPassed = false;
        for(var k in parts) {
            var p = parts[k].trim();
            if(p) {
                if(firstPassed) {
                    p = p.substr(0,1).toUpperCase() + p.substr(1);
                }
                else {
                    firstPassed = true;
                }
                camel += p;
            }
        }
        return camel;
    }
}
