
"use strict";

/*
 * @requires Ip4Address.js
 * @version 1.0.0
 */

/**
 * Utility functions to handle cookies.
 * @type type CookieQuestionCookieHelper
 */
class CookieQuestionCookieHelper {

    /**
     * Sets a cookie for the current domain, handles localhost too.
     * @param cookieName String The name of the cookie
     * @param cookieValue String The value of the cookie
     * @param validityDays Number The number of days the cookie should be valid
     * for. Optional, defaults to 365.
     * If negative, the cookie will be deleted immediately,
     * if zero the cookie will be a session cookie, .i.e. valid until the browser windows is closed
     * @param {string} domainName denotes the (sub-) domain to set the cookie 
     * for. Optional, if ungiven or empty, the cookie will be set for the host
     * including all subdomains, defaults to the empty string.
     * @param {boolean} forceOnLocalhost if true the cookie is set for localhost,
     * if false window.location.hostname ist tested to be a localhost ipadress
     * or "localhost". Optional, defaults to false.
     * @return {string} The set cookie value, an empty String on error
     * 
     * HINT: if Microsoft edge browser does not accept cookies please see
     * https://answers.microsoft.com/en-us/windows/forum/apps_windows_10-msedge/microsoft-edge-blocking-cookies-windows-10/3d865aca-a21e-445d-9804-d45a5d02efa2
     * 
     */
    static setCookieForDomain(cookieName = "", cookieValue = "", validityDays = 365, domainName = "", forceOnLocalhost = false) {
        var cookieStr = cookieName + '=' + cookieValue;
        var validityPart = (validityDays === 0) ? '' 
        : ('; expires=' + (new Date(((new Date()).getTime()) + (validityDays * 86400000 /* ms per day = 24 * 60 * 60 * 1000*/))).toGMTString());
        var domainPart = "";
        if (domainName || forceOnLocalhost) {
            var hostname = String(window.location.hostname);
            domainPart = "; domain="
                +
                ((forceOnLocalhost || Ip4Address.isIp4AddrInRange(hostname, [127, 0, 0, 0], [127, 255, 255, 255], 1) || (hostname === "::1") || (hostname.toLowerCase() === "localhost"))
                    ? "" : domainName
                );
        }
        var setCookieValue;
        try {
            document.cookie = cookieStr + validityPart + domainPart + "; path=/";
            setCookieValue = CookieQuestionCookieHelper.getCookieValue(cookieName);
        }
        catch (err) {
            setCookieValue = "";
        }
        return setCookieValue;
    }

    /**
     * Sets a session only cookie, valid until browser session ends.
     * @param cookieName String The name of the cookie
     * @param cookieValue String The value of the cookie
     * @param {string} domainName denotes the (sub-) domain to set the cookie 
     * for. Optional, if ungiven or empty, the cookie will be set for the host
     * including all subdomains, defaults to the empty string.
     * @return {string} The set cookie value, an empty String on error
     */
    static setSessionCookieForDomain(cookieName = "", cookieValue = "", domainName = "") {
        return CookieQuestionCookieHelper.setCookieForDomain(cookieName, cookieValue, 0, domainName);
    }

    /** 
     * Deletes the cookie named cookieName.
     * @param cookieName String The name of the cookie
     * @param {string} domainName denotes the (sub-) domain to set the cookie 
     * for. Optional, if ungiven or empty, the cookie will be rertieved for the 
     * host, valid for all subdomains, defaults to the empty string.
     */
    static deleteCookie(cookieName = "", domainName = "") {
        CookieQuestionCookieHelper.setCookieForDomain(cookieName, "", -1, domainName);
    }

    /**
     * Gets the value of a cookie, if set.
     * Hint: Assumes that the cookie of interest was set with a non-empty value.
     * @return: String The value of the cookie,
     * an empty String
     * if cookies are disabled
     * or no cookie with cookieName is stored
     * or there is a cookie with cookieName and it's value is the empty string.
     */
    static getCookieValue(cookieName = "") {
        var value = "";
        if (!!document.cookie) {
            var cookies = document.cookie.split('; ');
            var len = cookies.length;
            for (var i = 0; i < len; i++) {
                var keyValue = cookies[i].split('=');
                if (String(keyValue[0]) === cookieName) {
                    value = String(keyValue[1]);
                    break;
                }
            }
        }
        return value;
    }
}
